@@ -102,5 +102,8 @@ void sockaddr_to_str (const struct sockaddr *sa, int salen,
 		      size_t *plen);
 char *sockaddr_to_astr (const struct sockaddr *sa, int salen);
 
+/* urlconn.c */
+int url_connect (struct pies_url *url, struct grecs_sockaddr *source_addr);
+
 
 
