@@ -458,7 +458,7 @@ component_build_depmap (void)
 			_("component %s depends on %s, "
 			  "which is not declared"),
 			comp->tag, tag);
-		comp_array_remove (i);
+		comp_array[i]->flags |= CF_REMOVE;
 		continue;
 	      }
 	    depmap_set (depmap, i, tgt);
@@ -595,10 +595,9 @@ component_verify (struct component *comp, grecs_locus_t *locus)
 		     "%s", _("\"internal\" used with \"command\""));
 	}
     }
-  else if (!comp->argv)
-    COMPERR (grecs_error,
-	     "%s", _("missing command line"));
-
+  else if (!comp->command)
+    COMPERR (grecs_error, "%s", _("no 'command' statement"));
+  
   if (ISCF_TCPMUX (comp->flags))
     {
       comp->mode = pies_comp_inetd;
@@ -736,6 +735,12 @@ component_verify (struct component *comp, grecs_locus_t *locus)
 void
 component_finish (struct component *comp, grecs_locus_t *locus)
 {
+  if (component_verify (comp, locus))
+    {
+      component_free (comp);
+      return;
+    }
+
   if (comp->flags & CF_SHELL)
     {
       comp->argc = 3;
@@ -745,7 +750,7 @@ component_finish (struct component *comp, grecs_locus_t *locus)
       comp->argv[2] = grecs_strdup (comp->command);
       comp->argv[3] = NULL;
     }
-  else
+  else if (comp->command)
     {
       struct wordsplit ws;
       if (wordsplit (comp->command, &ws, WRDSF_DEFFLAGS))
@@ -766,29 +771,21 @@ component_finish (struct component *comp, grecs_locus_t *locus)
   if (comp->privs.groups)
     comp->privs.groups->cmp = list_str_cmp;
 
-  if (component_verify (comp, locus))
+  if (grecs_list_size (comp->prereq) == 1)
     {
-      component_free (comp);
-    }
-  else
-    {
-      size_t n = grecs_list_size (comp->prereq);
-      if (n == 1)
+      const char *item = grecs_list_index (comp->prereq, 0);
+      if (strcmp (item, "all") == 0)
 	{
-	  const char *item = grecs_list_index (comp->prereq, 0);
-	  if (strcmp (item, "all") == 0)
-	    {
-	      struct component *p;
+	  struct component *p;
 
-	      grecs_list_clear (comp->prereq);
-	      for (p = comp->prev; p; p = p->prev)
-		grecs_list_push (comp->prereq, grecs_strdup (comp->tag));
-	    }
-	  else if (strcmp (item, "none") == 0)
-	    {
-	      grecs_list_free (comp->prereq);
-	      comp->prereq = NULL;
-	    }
+	  grecs_list_clear (comp->prereq);
+	  for (p = comp->prev; p; p = p->prev)
+	    grecs_list_push (comp->prereq, grecs_strdup (comp->tag));
+	}
+      else if (strcmp (item, "none") == 0)
+	{
+	  grecs_list_free (comp->prereq);
+	  comp->prereq = NULL;
 	}
     }
 }
