@@ -1402,7 +1402,7 @@ check_stopping (struct prog *prog, time_t now)
     {
       if (prog->pid == 0)
 	logmsg (LOG_EMERG,
-		_("INTERNAL ERROR: attempting to kill unexisting process %s"),
+		_("INTERNAL ERROR: attempting to kill nonexistent process %s"),
 		prog_tag (prog));
       else if (prog->v.p.comp->flags & CF_SIGGROUP)
 	kill (-prog->pid, SIGKILL);
